; main.asm from rs_kbd project:
;  PIC initialization, variables declarations and a few generic routines.
; 
; Copyright 2016,2017 Mihai Gaitos, mihaig@hawk.ro
; 
; Permission is hereby granted, without written agreement and without
; license or royalty fees, to use, copy, modify, and distribute this
; software and its documentation for any purpose, provided that the
; above copyright notice and the following two paragraphs appear in
; all copies of this software.
; 
; IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
; DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
; ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
; IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
; DAMAGE.
; 
; THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
; BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
; FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
; ON AN "AS IS" BASIS, AND THE COPYRIGHT HOLDER HAS NO OBLIGATION TO
; PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.


W				EQU     H'0000'
F				EQU     H'0001'


;----- Register Files------------------------------------------------------

INDF            EQU     H'0000'
TMR0            EQU     H'0001'
PCL             EQU     H'0002'
STATUS          EQU     H'0003'
FSR             EQU     H'0004'
PORTA           EQU     H'0005'
PORTB           EQU     H'0006'
PCLATH			EQU		H'000A'
INTF			EQU     H'000B'
OPTION_REG      EQU     H'0001'
TRISA           EQU     H'0005'
TRISB           EQU     H'0006'
CMCON           EQU     H'001F'

T1CON			EQU		H'0010'
TMR1L			EQU		H'000E'
TMR1H			EQU		H'000F'
PIR1			EQU		H'000C'

TXSTA			EQU		H'18'
SPBRG			EQU		H'19'

RCSTA			EQU		H'18'
TXREG			EQU		H'19'
RCREG			EQU		H'1A'

;----- STATUS Bits --------------------------------------------------------
IRP                          EQU     H'0007'
RP1                          EQU     H'0006'
RP0                          EQU     H'0005'
NOT_TO                       EQU     H'0004'
NOT_PD                       EQU     H'0003'
Z                            EQU     H'0002'
DC                           EQU     H'0001'
C                            EQU     H'0000'

;==========================================================================
;
;       RAM Definition
;
;==========================================================================

    __MAXRAM H'01FF'
    __BADRAM H'07'-H'09', H'0D', H'13'-H'14', H'1B'-H'1E'
    __BADRAM H'87'-H'89', H'8D', H'8F'-H'91', H'93'-H'97', H'9E'
    __BADRAM H'105', H'107'-H'109', H'10C'-H'11F', H'150'-H'16F'
    __BADRAM H'185', H'187'-H'189', H'18C'-H'1EF'

;==========================================================================
;
;       Configuration Bits
;
;==========================================================================

_BODEN_ON                    EQU     H'3FFF'
_BODEN_OFF                   EQU     H'3FBF'
_CP_ALL                      EQU     H'03FF'
_CP_75                       EQU     H'17FF'
_CP_50                       EQU     H'2BFF'
_CP_OFF                      EQU     H'3FFF'
_DATA_CP_ON                  EQU     H'3EFF'
_DATA_CP_OFF                 EQU     H'3FFF'
_PWRTE_OFF                   EQU     H'3FFF'
_PWRTE_ON                    EQU     H'3FF7'
_WDT_ON                      EQU     H'3FFF'
_WDT_OFF                     EQU     H'3FFB'
_LVP_ON                      EQU     H'3FFF'
_LVP_OFF                     EQU     H'3F7F'
_MCLRE_ON                    EQU     H'3FFF'
_MCLRE_OFF                   EQU     H'3FDF'
_ER_OSC_CLKOUT               EQU     H'3FFF'
_ER_OSC_NOCLKOUT             EQU     H'3FFE'
_INTRC_OSC_CLKOUT            EQU     H'3FFD'
_INTRC_OSC_NOCLKOUT          EQU     H'3FFC'
_EXTCLK_OSC                  EQU     H'3FEF'
_LP_OSC                      EQU     H'3FEC'
_XT_OSC                      EQU     H'3FED'
_HS_OSC                      EQU     H'3FEE'

	__CONFIG        _BODEN_OFF & _CP_OFF & _DATA_CP_OFF & _PWRTE_OFF & _WDT_OFF & _LVP_OFF & _MCLRE_OFF & _INTRC_OSC_NOCLKOUT
;	__CONFIG		3F0B

;==========================================================================
;       Variable Definition
;==========================================================================
BYTE4ACK	EQU		H'20'	; last byte sent (in case keyboard requests resend)
NEXTKBB		EQU		H'21'	; next byte to send on ACK

CRTIN		EQU		H'22'	; current input (for encoder)
PRVI		EQU		H'23'	; previous (encoder) input - for change detect
EST			EQU		H'24'	; encoder "state machine" "position"
ENCR		EQU		H'25'	; encoder result

CRT_B		EQU		H'41'
TMP_B		EQU		H'42'	; USED for SERIAL RECEIVE

; SPEC_SC		EQU		H'43'	; Special SCANCODE (E0, E1, FA, not release -  it has its own flag)
CNT			EQU		H'44'	; generic counter, used in KBD, RS232 routines
CNT_KB		EQU		H'45'
TMP_SC		EQU		H'46'
KC			EQU		H'47'
MYFLAGS		EQU		H'48'

CNTD1		EQU		H'49'	; delay loop counter
CNTD2		EQU		H'4A'	; delay loop counter
CNT3		EQU		H'4B'

TMPC		EQU		H'4C'	; for printhex1
TMPS		EQU		H'4D'	; for DSEND
TMPH2		EQU		H'4E'	; for printhex2
TMPW		EQU		H'4F'	; generic


; CLINE		EQU		H'50'	; was used for LCD display
; CCOL		EQU		H'51'	; same

CNTKBW		EQU		H'52'
; CCOLT		EQU		H'53'
; CCOLB		EQU		H'54'

CRT_OP		EQU		H'55'
K_FLAGS		EQU		H'56'



; ****************************
; *** CONSTANTS DEFINITION ***
; ****************************


; ENCODER related

ENCMASK		EQU		B'00110000'		; PORT bits where encoder is connected
ENCBASE		EQU		H'600'			; encoder "state machine" address in program memory
ENC00		EQU		B'00000000'		; bits for encoder position 00
ENC01		EQU		B'00010000'		;	..
ENC10		EQU		B'00100000'		;	..
ENC11		EQU		B'00110000'		; to 11

; MY_FLAGS bits
MFTMPBIT	EQU		D'4'	; for now used to XOR Numlock and Shift
KBTOSEND	EQU		D'5'	; SET in MYFLAGS if there is one more BYTE to send to keyboard
KBPAR		EQU		D'6'	; keyboard parity bit
KBWERR		EQU		D'7'	; bit7 used to signal an error while waiting for keyboard

; K_FLAGS bits
F_SHIFT		EQU		D'0'
F_NUMLOCK	EQU		D'1'	; NUMLOCK and CAPSLOCK
F_CAPSLOCK	EQU		D'2'	; MUST NOT BE CHANGED! they correspond to the actual keyboard command bits
F_CTRL		EQU		D'3'
F_ALT		EQU		D'4'
F_SPECIAL	EQU		D'5'
F_RELEASE	EQU		D'6'

OP_ASCII	EQU		D'1'
OP_HEX		EQU		D'2'
; OP_NUM		EQU		D'3'	; no longer used

; pins/bits
; PORTB
; 4 & 5 are used for the encoder, 1 & 2 are RX & TX
K_CLK		EQU		D'3'
K_DAT		EQU		D'0'
; R&S specific:
LRLED		EQU		D'6'	; Left - Right LED - active when Shift is up (not pressed)
UDLED		EQU		D'7'	; Up - Down LED - active when Shift is down (pressed)


; PORTA
RDLED		EQU		D'3'	; "Ready" LED - mainly for debug

		ORG	0			;Reset vector address
		GOTO	RESET	;goto RESET routine on boot.
		ORG 4			;Interrupt vector


;      	**********************************
;       **  RESET :  main boot routine  **
;       **********************************

RESET	MOVLW	B'00000111'	;Disable Comparator module
		MOVWF	CMCON
		BSF	STATUS,RP0	;Switch to register bank 1, for OPTION_REG
					;7		= Enable pull-ups
					;6		= INT on rising edge
					;6		= TMR0 to CLKOUT
					;4		= TMR0 Incr low2high trans.
					;3		= Prescaler assign to Timer0
					;2-0	= Prescaler rate is 1:256
		MOVLW	B'01010111'	;Set PIC options (See above).
		MOVWF	OPTION_REG	;Write the OPTION register.
		;
		MOVLW	B'01010000'	; Disable interrupts, we can enable them later
		MOVWF	INTF		; with BSF INTCON,7


; ***** COMPUTE BAUD RATE; JUST ONE RATE FOR NOW *****

		MOVLW	D'26'		; FOR 2400 BAUD
		MOVWF	SPBRG		; baud generator

		MOVLW	B'00100000'	; SET TXEN bit
		MOVWF	TXSTA

		BCF		STATUS,RP0	;Switch Back to reg. Bank 0

		MOVLW	B'10010000'	; SET SPEN,CREN bits
		MOVWF	RCSTA

		CLRF	PORTB
		CLRF	PORTA
		CLRF	MYFLAGS

		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'11111011';RB control (0=OUTPUT,1=INPUT)
					; RB0 and RB3 will be used for KBD (DATA and CLOCK)
					; RB1,RB2 are serial RX,TX
		MOVWF	TRISB
		MOVLW	B'11110111';RA control (0=OUTPUT,1=INPUT)
					; RA3 = READY LED
		MOVWF	TRISA
		BCF	STATUS,RP0
		CLRF	PORTA

		CALL	ENC_INIT		; initialize encoder receiver variables

; There was a time when we proudly announced getting here :)
;		MOVLW	'R'
;		CALL	SENDCHR
;		MOVLW	'e'
;		CALL	SENDCHR
;		MOVLW	'a'
;		CALL	SENDCHR
;		MOVLW	'd'
;		CALL	SENDCHR
;		MOVLW	'y'
;		CALL	SENDCHR
;		MOVLW	'.'
;		CALL	SENDCHR
;		MOVLW	'.'
;		CALL	SENDCHR
;		MOVLW	'.'
;		CALL	SENDCHR

		MOVLW	OP_ASCII
		MOVWF	CRT_OP
		
		GOTO	INITKBD	; in kbd.asm; the main loop is there

; mode procedures - now only used for debug
CHGMODE_TO_ASCII
		MOVLW	OP_ASCII
		MOVWF	CRT_OP
		MOVLW	'A'
		CALL	SENDCHR
		MOVLW	'S'
		CALL	SENDCHR
		MOVLW	'C'
		CALL	SENDCHR
		MOVLW	'I'
		CALL	SENDCHR
		MOVLW	'I'
		CALL	SENDCHR
		MOVLW	' '
		CALL	SENDCHR
		MOVLW	'M'
		CALL	SENDCHR
		MOVLW	'O'
		CALL	SENDCHR
		MOVLW	'D'
		CALL	SENDCHR
		MOVLW	'E'
		CALL	SENDCHR
		MOVLW	D'13'
		CALL	SENDCHR
		MOVLW	D'10'
		CALL	SENDCHR
		RETURN		
CHGMODE_TO_HEX
		MOVLW	OP_HEX
		MOVWF	CRT_OP
		MOVLW	'H'
		CALL	SENDCHR
		MOVLW	'E'
		CALL	SENDCHR
		MOVLW	'X'
		CALL	SENDCHR
		MOVLW	' '
		CALL	SENDCHR
		MOVLW	'M'
		CALL	SENDCHR
		MOVLW	'O'
		CALL	SENDCHR
		MOVLW	'D'
		CALL	SENDCHR
		MOVLW	'E'
		CALL	SENDCHR
		MOVLW	D'13'
		CALL	SENDCHR
		MOVLW	D'10'
		CALL	SENDCHR
		RETURN


; RS232 procedures
SENDCHR
		BSF	STATUS,RP0	; TXSTA is on bank 1
SENDWAIT
		BTFSS	TXSTA,1	; wait for TX ready (in case the previous TX is not finished yet)
		GOTO	SENDWAIT
		BCF	STATUS,RP0	; back to bank 0
		MOVWF	TXREG
		RETURN

; not really RS232, but used here

PRNHEX1
		ANDLW   H'0F'   ; only lower nibble
		MOVWF   TMPC
		SUBLW   D'9'    ; if carry is 0 then W>9
		BTFSC   STATUS,C
		GOTO    PRNH1N
		MOVLW   D'55'
		ADDWF   TMPC,0          ; W>0, W+55=corresponding hex letter
		CALL    SENDCHR
		RETURN
PRNH1N
		MOVLW   H'30'
		ADDWF   TMPC,0
		CALL    SENDCHR
		RETURN


PRNHEX2
		MOVWF	TMPH2
		SWAPF   TMPH2,W
		CALL    PRNHEX1
		MOVF    TMPH2,W
		CALL    PRNHEX1
		RETURN


DELAY_MID
		MOVLW	D'1'
		MOVWF	CNTD1
		MOVLW	D'20'
		MOVWF	CNTD2
		GOTO	DLOOP

DELAY_BIG
		MOVLW	D'0'
		MOVWF	CNTD1
		MOVLW	D'0'
		MOVWF	CNTD2
		GOTO	DLOOP


DELAY_100
		MOVLW	D'40'	; inner
		MOVWF	CNTD1
		MOVLW	D'1'	; outer
		MOVWF	CNTD2
DLOOP
		CALL	DRTRN
		DECFSZ	CNTD1,1
		GOTO	DLOOP
		DECFSZ	CNTD2,1
		GOTO	DLOOP
		RETURN


DRTRN
		RETURN

		include "encoder.asm"
		include	"ktable.asm"
		include	"kbd.asm"
	END
